@@ -419,7 +419,7 @@ public:
 protected:
     virtual void Init() override;
 
-    void ConfigurePre(SRTSOCKET socket);
+    void ConfigurePre();
     void ConfigurePost(SRTSOCKET socket);
 
     using Medium::Error;
@@ -448,6 +448,32 @@ public:
 
     TcpMedium(UriParser u, size_t ch): Medium(u, ch) {}
 
+#endif
+
+#ifdef _WIN32
+    static int tcp_close(int socket)
+    {
+        return ::closesocket(socket);
+    }
+
+    enum { DEF_SEND_FLAG = 0 };
+
+#elif defined(LINUX) || defined(GNU)
+    static int tcp_close(int socket)
+    {
+        return ::close(socket);
+    }
+
+    enum { DEF_SEND_FLAG = MSG_NOSIGNAL };
+
+#elif defined(BSD) || defined(OSX) || (TARGET_OS_IOS == 1) || (TARGET_OS_TV == 1)
+    static int tcp_close(int socket)
+    {
+        return ::close(socket);
+    }
+
+    enum { DEF_SEND_FLAG = 0 };
+
 #endif
 
     bool IsOpen() override { return m_open; }
@@ -460,7 +486,7 @@ public:
         lock_guard<mutex> lk(access);
         if (m_socket == -1)
             return;
-        ::close(m_socket);
+        tcp_close(m_socket);
         m_socket = -1;
     }
 
@@ -475,9 +501,12 @@ public:
 
 protected:
 
-    // Just models. No options are predicted for now.
-    void ConfigurePre(int )
+    void ConfigurePre()
     {
+#if defined(__APPLE__)
+        int optval = 1;
+        setsockopt(m_socket, SOL_SOCKET, SO_NOSIGPIPE, &optval, sizeof(optval));
+#endif
     }
 
     void ConfigurePost(int)
@@ -516,11 +545,11 @@ void SrtMedium::Init()
     m_options["transtype"] = "file";
 }
 
-void SrtMedium::ConfigurePre(SRTSOCKET so)
+void SrtMedium::ConfigurePre()
 {
     vector<string> fails;
     m_options["mode"] = "caller";
-    SrtConfigurePre(so, "", m_options, &fails);
+    SrtConfigurePre(m_socket, "", m_options, &fails);
     if (!fails.empty())
     {
         cerr << "Failed options: " << Printable(fails) << endl;
@@ -543,7 +572,7 @@ void SrtMedium::CreateListener()
 
     m_socket = srt_create_socket();
 
-    ConfigurePre(m_socket);
+    ConfigurePre();
 
     sockaddr_in sa = CreateAddrInet(m_uri.host(), m_uri.portno());
 
@@ -570,7 +599,7 @@ void TcpMedium::CreateListener()
     int backlog = 5; // hardcoded!
 
     m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    ConfigurePre(m_socket);
+    ConfigurePre();
 
     sockaddr_in sa = CreateAddrInet(m_uri.host(), m_uri.portno());
 
@@ -578,14 +607,14 @@ void TcpMedium::CreateListener()
 
     if (stat == -1)
     {
-        close(m_socket);
+        tcp_close(m_socket);
         Error(errno, "bind");
     }
 
     stat = listen(m_socket, backlog);
     if ( stat == -1 )
     {
-        close(m_socket);
+        tcp_close(m_socket);
         Error(errno, "listen");
     }
 
@@ -628,7 +657,7 @@ unique_ptr<Medium> TcpMedium::Accept()
 void SrtMedium::CreateCaller()
 {
     m_socket = srt_create_socket();
-    ConfigurePre(m_socket);
+    ConfigurePre();
 
     // XXX setting up outgoing port not supported
 }
@@ -636,7 +665,7 @@ void SrtMedium::CreateCaller()
 void TcpMedium::CreateCaller()
 {
     m_socket = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    ConfigurePre(m_socket);
+    ConfigurePre();
 }
 
 void SrtMedium::Connect()
@@ -671,7 +700,7 @@ int SrtMedium::ReadInternal(char* buffer, int size)
 
 int TcpMedium::ReadInternal(char* buffer, int size)
 {
-    return read(m_socket, buffer, size);
+    return ::recv(m_socket, buffer, size, 0);
 }
 
 bool SrtMedium::IsErrorAgain()
@@ -779,7 +808,7 @@ void TcpMedium::Write(ref_t<bytevector> r_buffer)
 {
     bytevector& buffer = *r_buffer;
 
-    int st = ::write(m_socket, buffer.data(), buffer.size());
+    int st = ::send(m_socket, buffer.data(), buffer.size(), DEF_SEND_FLAG);
     if (st == -1)
     {
         Error(errno, "send");
@@ -948,6 +977,12 @@ int OnINT_StopService(int)
 
 int main( int argc, char** argv )
 {
+    if (!SysInitializeNetwork())
+    {
+        cerr << "Fail to initialize network module.";
+        return 1;
+    }
+
     size_t chunk = default_chunk;
 
     set<string>
