@@ -34,28 +34,20 @@ written by
 int HaiCrypt_Tx_GetBuf(HaiCrypt_Handle hhc, size_t data_len, unsigned char **in_pp)
 {
 	hcrypt_Session *crypto = (hcrypt_Session *)hhc;
-	int pad_factor = (HCRYPT_CTX_MODE_AESECB == crypto->ctx->mode ? 128/8 : 1);
 
 	ASSERT(NULL != crypto);
-	ASSERT(NULL != crypto->cipher);
+	ASSERT(NULL != crypto->cryspr);
+
+	int pad_factor = (HCRYPT_CTX_MODE_AESECB == crypto->ctx->mode ? 128/8 : 1);
 
-	if (NULL != crypto->cipher->getinbuf) {
-		ASSERT(NULL != crypto->cipher_data);
-		if (0 >= crypto->cipher->getinbuf(crypto->cipher_data, crypto->msg_info->pfx_len, 
-			data_len, pad_factor, in_pp)) {
-			*in_pp = NULL;
-			return(-1);
-		}
-	} else {
 #ifndef _WIN32
-		ASSERT(crypto->inbuf != NULL);
+	ASSERT(crypto->inbuf != NULL);
 #endif
-		size_t in_len = crypto->msg_info->pfx_len + hcryptMsg_PaddedLen(data_len, pad_factor);
-		*in_pp = crypto->inbuf;
-		if (in_len > crypto->inbuf_siz) {
-			*in_pp = NULL;
-			return(-1);
-		}
+	size_t in_len = crypto->msg_info->pfx_len + hcryptMsg_PaddedLen(data_len, pad_factor);
+	*in_pp = crypto->inbuf;
+	if (in_len > crypto->inbuf_siz) {
+		*in_pp = NULL;
+		return(-1);
 	}
 	return(crypto->msg_info->pfx_len);
 }
@@ -122,8 +114,8 @@ int HaiCrypt_Tx_Data(HaiCrypt_Handle hhc,
 		indata.payload  = in_data;
 		indata.len      = in_len;
 
-		if (0 > (nbout = crypto->cipher->encrypt(crypto->cipher_data, ctx, &indata, 1, NULL, NULL, NULL))) {
-			HCRYPT_LOG(LOG_ERR, "%s", "encrypt failed\n");
+		if (0 > (nbout = crypto->cryspr->ms_encrypt(crypto->cryspr_cb, ctx, &indata, 1, NULL, NULL, NULL))) {
+			HCRYPT_LOG(LOG_ERR, "%s", "ms_encrypt failed\n");
 			return(nbout);
 		}
 	}
@@ -170,8 +162,8 @@ int HaiCrypt_Tx_Process(HaiCrypt_Handle hhc,
 		indata.payload  = &in_msg[ctx->msg_info->pfx_len];
 		indata.len      = in_len - ctx->msg_info->pfx_len;
 
-		if (crypto->cipher->encrypt(crypto->cipher_data, ctx, &indata, 1, &out_p[nbout], &out_len_p[nbout], &nb)) {
-			HCRYPT_LOG(LOG_ERR, "%s", "encrypt failed\n");
+		if (crypto->cryspr->ms_encrypt(crypto->cryspr_cb, ctx, &indata, 1, &out_p[nbout], &out_len_p[nbout], &nb)) {
+			HCRYPT_LOG(LOG_ERR, "%s", "ms_encrypt failed\n");
 			return(nbout);
 		}
 	}
